;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
;+
; NAME:curl
;
; PURPOSE:calcule la composante verticale du rotationnel d''un champ
; de vecteur horizontaux
;
; CATEGORY:calcule sur les matrices
;
; CALLING SEQUENCE:res=curl(u,v)
;
; INPUTS:
;       u et v deux matrices representant les coordonnes d''un
;       champ de vecteur
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:res: une matrice 2d
;
; COMMON BLOCKS:
;	common.pro
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
; les matrices u et v peuvent de 2 a 4 dimensions.
; attention pour distinger les differents configurations de u et v
; (xy, xyz, xyt, xyzt), on regarde la variable du common 
;        -time qui contient le calendrier en jour julien d''IDL auquel
;        se rapportent u et v ansi que la variable 
;        -jpt qui est le nombre de pas de temps a considerer ds time.
; les tableaux u et v sont decoupes sur le meme domaine
; geographique. A cause du decalage des grilles T, U, V et F il est
; possiible que ces 2 tableaux n''aient pas la meme taille et se
; repportent a des indices differents. Si tel est le cas les tableaux
; sont redecoupes sur les indices qu'ils ont en commun et le dommaine
; est redefinit pour qu'il colle a ces indices communs.
; pour eviter ces redecoupes utiliser le mot cles /memeindice ds
; domdef.pro
;
;
; les points sur le bord du dessin sont mis a !values.f_nan 
;
; EXAMPLE:
;
; MODIFICATION HISTORY:Guillaume Roullet (grlod@ipsl.jussieu.fr)
; 
;                      Sebastien Masson (smasson@lodyc.jussieu.fr)
;                      adaptation pour marcher avec un domaine reduit
;
;                      21/5/1999: valeurs manquantes a !values.f_nan
;periodicite 
;-
;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
FUNCTION curl, uu, vv
@common
   tempsun = systime(1)         ; pour key_performance
;
   u = litchamp(uu)
   v = litchamp(vv)

   date1 = time[0]
   if n_elements(jpt) EQ 0 then date2 = date1 ELSE date2 = time[jpt-1]
   if (size(u))[0] NE (size(v))[0] then return,  -1

;------------------------------------------------------------
; on trouve les points que u et v ont en communs
;------------------------------------------------------------
   indicexu = (lindgen(jpi))[premierxu:premierxu+nxu-1]
   indicexv = (lindgen(jpi))[premierxv:premierxv+nxv-1]
   indicex = inter(indicexu, indicexv)
   indiceyu = (lindgen(jpj))[premieryu:premieryu+nyu-1]
   indiceyv = (lindgen(jpj))[premieryv:premieryv+nyv-1]
   indicey = inter(indiceyu, indiceyv)
   nx = n_elements(indicex) 
   ny = n_elements(indicey)
   case 1 of
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;xyz
;----------------------------------------------------------------------------
      (size(u))[0] EQ 3 AND date1 EQ date2 :BEGIN 
         indice2d = lindgen(jpi, jpj)
         indice2d = indice2d[indicex[0]:indicex[0]+nx-1,indicey[0]:indicey[0]+ny-1]
;------------------------------------------------------------
; extraction de u et v sur le domaine qui convient
;------------------------------------------------------------
         case 1 of
            (size(u))[0] NE 3 OR (size(v))[0] NE 3: return,  -1
            (size(u))[1] EQ nxu AND (size(u))[2] EQ nyu AND $
             (size(v))[1] EQ nxv AND (size(v))[2] EQ nyv:BEGIN 
            case 1 of
               nxu NE nx:if indicex[0] EQ premierxu then u = u[0:nx-1, *, *] ELSE u = u[1: nx, *, *]
               nxv NE nx:if indicex[0] EQ premierxv then v = v[0:nx-1, *, *] ELSE v = v[1: nx, *, *]
               nyu NE ny:if indicey[0] EQ premieryu then u = u[*, 0:ny-1, *] ELSE u = u[*, 1: ny, *]
               nyv NE ny:if indicey[0] EQ premieryv then v = v[*, 0:ny-1, *] ELSE v = v[*, 1: ny, *]
               ELSE :
            endcase
            END
            (size(u))[1] EQ jpi AND (size(u))[2] EQ jpj AND $
             (size(v))[1] EQ jpi AND (size(v))[2] EQ jpj:BEGIN 
               u = u[indicex[0]:indicex[0]+nx-1,indicey[0]:indicey[0]+ny-1, *]
               v = v[indicex[0]:indicex[0]+nx-1,indicey[0]:indicey[0]+ny-1, *]
            END
            ELSE:return,  -1
         endcase
;------------------------------------------------------------
; calcul du rotationnel
;------------------------------------------------------------
         coefu = (e1u[indice2d])[*]#replicate(1, nzt)
         coefu = reform(coefu, nx, ny, nzt, /over)
         coefu = coefu*(umask())[indicex[0]:indicex[0]+nx-1,indicey[0]:indicey[0]+ny-1, premierzt:dernierzt]
         terreu = where(coefu EQ 0)
         if terreu[0] NE -1 then coefu[temporary(terreu)] = !values.f_nan
         
         coefv = (e2v[indice2d])[*]#replicate(1, nzt)
         coefv = reform(coefv, nx, ny, nzt, /over)
         coefv = coefv*(vmask())[indicex[0]:indicex[0]+nx-1,indicey[0]:indicey[0]+ny-1, premierzt:dernierzt]
         terrev = where(coefv EQ 0)
         if terrev[0] NE -1 then coefv[temporary(terrev)] = !values.f_nan

         tabf = (fmask())[indicex[0]:indicex[0]+nx-1,indicey[0]:indicey[0]+ny-1, premierzt:dernierzt]
         div = (e1f[indice2d]*e2f[indice2d])[*]#replicate(1, nzt)
         div = reform(div, nx, ny, nzt, /over)
         tabf = tabf/div
;
         zu = u*temporary(coefu)
         zv = v*temporary(coefv)

         psi =  (shift(zv, -1, 0, 0)-zv) + (zu-shift(zu, 0, -1, 0))
         psi = tabf*psi
;------------------------------------------------------------
; mise a !values.f_nan de la bordure
;------------------------------------------------------------
         if NOT keyword_set(key_periodique)  OR nx NE jpi then begin
            psi(0, *, *) = !values.f_nan
            psi(nx-1, *, *) = !values.f_nan
         endif
         psi(*, 0, *) = !values.f_nan
         psi(*, ny-1, *) = !values.f_nan
;
         if n_elements(valmask) EQ 0 THEN valmask = 1e20
         terref =  where(tabf EQ 0)
         if terref[0] NE -1 then psi[temporary(terref)] = valmask
;------------------------------------------------------------
; pour le trace graphique
;------------------------------------------------------------
         domdef, (glamt[indice2d])[0, 0], (glamu[indice2d])[nx-1, 0],(gphit[indice2d])[0, 0], (gphiv[indice2d])[0, ny-1], prof1, prof2, grille = ['t', 'f']
         if keyword_set(direc) then psi = moyenne(psi,direc,/nan, boite = boite)

;----------------------------------------------------------------------------
      END
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;xyt
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
      date1 NE date2 AND (size(u))[0] EQ 3 :BEGIN 
         indice2d = lindgen(jpi, jpj)
         indice2d = indice2d[indicex[0]:indicex[0]+nx-1,indicey[0]:indicey[0]+ny-1]
;------------------------------------------------------------
; extraction de u et v sur le domaine qui convient
;------------------------------------------------------------
         case 1 of
            (size(u))[1] EQ nxu AND (size(u))[2] EQ nyu AND $
             (size(v))[1] EQ nxv AND (size(v))[2] EQ nyv:BEGIN 
               if nxu NE nx then $
                if indicex[0] EQ premierxu then u = u[0:nx-1, *, *] ELSE u = u[1: nx, *, *] 
               IF nxv NE nx THEN $
                if indicex[0] EQ premierxv then v = v[0:nx-1, *, *] ELSE v = v[1: nx, *, *]
               IF nyu NE ny THEN $
                if indicey[0] EQ premieryu then u = u[*, 0:ny-1, *] ELSE u = u[*, 1: ny, *] 
               IF nyv NE ny THEN $
                if indicey[0] EQ premieryv then v = v[*, 0:ny-1, *] ELSE v = v[*, 1: ny, *]
            END
            (size(u))[1] EQ jpi AND (size(u))[2] EQ jpj AND $
             (size(v))[1] EQ jpi AND (size(v))[2] EQ jpj:BEGIN 
               u = u[indicex[0]:indicex[0]+nx-1,indicey[0]:indicey[0]+ny-1, *]
               v = v[indicex[0]:indicex[0]+nx-1,indicey[0]:indicey[0]+ny-1, *]
            END
            ELSE:BEGIN 
               print, 'problemes d''adequation entre la taille du domaine et la taille des matrices necessaires a tracer des vecteurs'
               return, -1
            end
         endcase
;----------------------------------------------------------------------------
; calcul du rotationnel
;----------------------------------------------------------------------------
         coefu = e1u[indice2d]*(umask())[indice2d+jpi*jpj*premierzt]
         terreu = where(coefu EQ 0)
         if terreu[0] NE -1 then coefu[temporary(terreu)] = !values.f_nan
         coefu = temporary(coefu[*])#replicate(1, jpt)
         coefu = reform(coefu, nx, ny, jpt, /over)
;
         coefv = e2v[indice2d]*(vmask())[indice2d+jpi*jpj*premierzt]
         terrev = where(coefv EQ 0)
         if terrev[0] NE -1 then coefv[temporary(terrev)] = !values.f_nan
         coefv = temporary(coefv[*])#replicate(1, jpt)
         coefv = reform(coefv, nx, ny, jpt, /over)
;
         tabf = (fmask())[indice2d+jpi*jpj*premierzt]/(e1f[indice2d]*e2f[indice2d])
         tabf = temporary(tabf[*])#replicate(1, jpt)
         tabf = reform(tabf, nx, ny, jpt, /over)
;------------------------------------------------------------
; calcul du rotationnel
;------------------------------------------------------------
         zu = u*temporary(coefu)
         zv = v*temporary(coefv)
;
         psi =  (shift(zv, -1, 0, 0)-zv) + (zu-shift(zu, 0, -1, 0))
         psi = tabf*psi
;------------------------------------------------------------
; mise a !values.f_nan de la bordure
;------------------------------------------------------------
         if NOT keyword_set(key_periodique) OR nx NE jpi then begin
            psi(0, *, *) = !values.f_nan
            psi(nx-1, *, *) = !values.f_nan
         endif
         psi(*, 0, *) = !values.f_nan
         psi(*, ny-1, *) = !values.f_nan
         if n_elements(valmask) EQ 0 THEN valmask = 1e20
         terref =  where(tabf EQ 0)
         if terref[0] NE -1 then psi[temporary(terref)] = valmask
;----------------------------------------------------------------------------
         domdef, (glamt[indice2d])[0, 0], (glamu[indice2d])[nx-1, 0],(gphit[indice2d])[0, 0], (gphiv[indice2d])[0, ny-1], prof1, prof2, grille = ['t', 'f']
         if keyword_set(direc) then psi = grossemoyenne(psi,direc,/nan, boite = boite)
;----------------------------------------------------------------------------
      END
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;xyzt
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
      date1 NE date2 AND (size(u))[0] EQ 4:BEGIN 
         return, report('non code!')
      END
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;xy
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
      ELSE:BEGIN                ;xy
         indice2d = lindgen(jpi, jpj)
         indice2d = indice2d[indicex[0]:indicex[0]+nx-1,indicey[0]:indicey[0]+ny-1]
;------------------------------------------------------------
; extraction de u et v sur le domaine qui convient
;------------------------------------------------------------
         case 1 of
            (size(u))[0] NE 2 OR (size(v))[0] NE 2: return,  -1
            (size(u))[1] EQ nxu AND (size(u))[2] EQ nyu AND $
             (size(v))[1] EQ nxv AND (size(v))[2] EQ nyv:BEGIN 
               if nxu NE nx then $
                if indicex[0] EQ premierxu then u = u[0:nx-1, *] ELSE u = u[1: nx, *] 
               IF nxv NE nx THEN $
                if indicex[0] EQ premierxv then v = v[0:nx-1, *] ELSE v = v[1: nx, *]
               IF nyu NE ny THEN $
                if indicey[0] EQ premieryu then u = u[*, 0:ny-1] ELSE u = u[*, 1: ny] 
               IF nyv NE ny THEN $
                if indicey[0] EQ premieryv then v = v[*, 0:ny-1] ELSE v = v[*, 1: ny]
            END
            (size(u))[1] EQ jpi AND (size(u))[2] EQ jpj AND $
             (size(v))[1] EQ jpi AND (size(v))[2] EQ jpj:BEGIN 
               u = u[indice2d]
               v = v[indice2d]
            END
            ELSE:return,  -1
         endcase
;------------------------------------------------------------
; calcul du rotationnel
;------------------------------------------------------------
         coefu = e1u[indice2d]*(umask())[indice2d+jpi*jpj*premierzt]
         terreu = where(coefu EQ 0)
         if terreu[0] NE -1 then coefu[temporary(terreu)] = !values.f_nan
         coefv = e2v[indice2d]*(vmask())[indice2d+jpi*jpj*premierzt]
         terrev = where(coefv EQ 0)
         if terrev[0] NE -1 then coefv[temporary(terrev)] = !values.f_nan
         tabf = (fmask())[indice2d+jpi*jpj*premierzt]/(e1f[indice2d]*e2f[indice2d])
;
         zu = u*temporary(coefu)
         zv = v*temporary(coefv)

         psi =  (shift(zv, -1, 0)-zv) + (zu-shift(zu, 0, -1))
         psi = tabf*psi

;------------------------------------------------------------
; mise a !values.f_nan de la bordure
;------------------------------------------------------------
         if  NOT keyword_set(key_periodique) OR nx NE jpi then begin
            psi(0, *) = !values.f_nan
            psi(nx-1, *) = !values.f_nan
         endif
         psi(*, 0) = !values.f_nan
         psi(*, ny-1) = !values.f_nan
;
         if n_elements(valmask) EQ 0 THEN valmask = 1e20
         terref =  where(tabf EQ 0)
         if terref[0] NE -1 then psi[temporary(terref)] = valmask
;------------------------------------------------------------
; pour le trace graphique
;------------------------------------------------------------
         domdef, (glamt[indice2d])[0, 0], (glamf[indice2d])[nx-1, 0],(gphit[indice2d])[0, 0], (gphif[indice2d])[0, ny-1], prof1, prof2, grille = ['t', 'f']
         if keyword_set(direc) then psi = moyenne(psi,direc,/nan, boite = boite)


      END
;----------------------------------------------------------------------------
   endcase
;------------------------------------------------------------
   if keyword_set(key_performance) THEN print, 'temps curl', systime(1)-tempsun 
;------------------------------------------------------------
   vargrid = 'F'
   varname = 'vorticity'
   return, psi
end
